# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.dialogflowcx_v3.types import flow as gcdc_flow
from google.cloud.dialogflowcx_v3.types import intent as gcdc_intent
from google.cloud.dialogflowcx_v3.types import page as gcdc_page
from google.cloud.dialogflowcx_v3.types import response_message
from google.cloud.dialogflowcx_v3.types import session
from google.cloud.dialogflowcx_v3.types import transition_route_group
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import struct_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.dialogflow.cx.v3",
    manifest={
        "TestResult",
        "TestCase",
        "TestCaseResult",
        "TestConfig",
        "ConversationTurn",
        "TestRunDifference",
        "TransitionCoverage",
        "TransitionRouteGroupCoverage",
        "IntentCoverage",
        "CalculateCoverageRequest",
        "CalculateCoverageResponse",
        "ListTestCasesRequest",
        "ListTestCasesResponse",
        "BatchDeleteTestCasesRequest",
        "CreateTestCaseRequest",
        "UpdateTestCaseRequest",
        "GetTestCaseRequest",
        "RunTestCaseRequest",
        "RunTestCaseResponse",
        "RunTestCaseMetadata",
        "BatchRunTestCasesRequest",
        "BatchRunTestCasesResponse",
        "BatchRunTestCasesMetadata",
        "TestError",
        "ImportTestCasesRequest",
        "ImportTestCasesResponse",
        "ImportTestCasesMetadata",
        "TestCaseError",
        "ExportTestCasesRequest",
        "ExportTestCasesResponse",
        "ExportTestCasesMetadata",
        "ListTestCaseResultsRequest",
        "ListTestCaseResultsResponse",
        "GetTestCaseResultRequest",
    },
)


class TestResult(proto.Enum):
    r"""The test result for a test case and an agent environment."""
    TEST_RESULT_UNSPECIFIED = 0
    PASSED = 1
    FAILED = 2


class TestCase(proto.Message):
    r"""Represents a test case.
    Attributes:
        name (str):
            The unique identifier of the test case.
            [TestCases.CreateTestCase][google.cloud.dialogflow.cx.v3.TestCases.CreateTestCase]
            will populate the name automatically. Otherwise use format:
            ``projects/<Project ID>/locations/<LocationID>/agents/ <AgentID>/testCases/<TestCase ID>``.
        tags (Sequence[str]):
            Tags are short descriptions that users may
            apply to test cases for organizational and
            filtering purposes. Each tag should start with
            "#" and has a limit of 30 characters.
        display_name (str):
            Required. The human-readable name of the test
            case, unique within the agent. Limit of 200
            characters.
        notes (str):
            Additional freeform notes about the test
            case. Limit of 400 characters.
        test_config (google.cloud.dialogflowcx_v3.types.TestConfig):
            Config for the test case.
        test_case_conversation_turns (Sequence[google.cloud.dialogflowcx_v3.types.ConversationTurn]):
            The conversation turns uttered when the test
            case was created, in chronological order. These
            include the canonical set of agent utterances
            that should occur when the agent is working
            properly.
        creation_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. When the test was created.
        last_test_result (google.cloud.dialogflowcx_v3.types.TestCaseResult):
            The latest test result.
    """

    name = proto.Field(proto.STRING, number=1,)
    tags = proto.RepeatedField(proto.STRING, number=2,)
    display_name = proto.Field(proto.STRING, number=3,)
    notes = proto.Field(proto.STRING, number=4,)
    test_config = proto.Field(proto.MESSAGE, number=13, message="TestConfig",)
    test_case_conversation_turns = proto.RepeatedField(
        proto.MESSAGE, number=5, message="ConversationTurn",
    )
    creation_time = proto.Field(
        proto.MESSAGE, number=10, message=timestamp_pb2.Timestamp,
    )
    last_test_result = proto.Field(proto.MESSAGE, number=12, message="TestCaseResult",)


class TestCaseResult(proto.Message):
    r"""Represents a result from running a test case in an agent
    environment.

    Attributes:
        name (str):
            The resource name for the test case result. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/testCases/ <TestCase ID>/results/<TestCaseResult ID>``.
        environment (str):
            Environment where the test was run. If not
            set, it indicates the draft environment.
        conversation_turns (Sequence[google.cloud.dialogflowcx_v3.types.ConversationTurn]):
            The conversation turns uttered during the
            test case replay in chronological order.
        test_result (google.cloud.dialogflowcx_v3.types.TestResult):
            Whether the test case passed in the agent
            environment.
        test_time (google.protobuf.timestamp_pb2.Timestamp):
            The time that the test was run.
    """

    name = proto.Field(proto.STRING, number=1,)
    environment = proto.Field(proto.STRING, number=2,)
    conversation_turns = proto.RepeatedField(
        proto.MESSAGE, number=3, message="ConversationTurn",
    )
    test_result = proto.Field(proto.ENUM, number=4, enum="TestResult",)
    test_time = proto.Field(proto.MESSAGE, number=5, message=timestamp_pb2.Timestamp,)


class TestConfig(proto.Message):
    r"""Represents configurations for a test case.
    Attributes:
        tracking_parameters (Sequence[str]):
            Session parameters to be compared when
            calculating differences.
        flow (str):
            Flow name. If not set, default start flow is assumed.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/flows/<Flow ID>``.
    """

    tracking_parameters = proto.RepeatedField(proto.STRING, number=1,)
    flow = proto.Field(proto.STRING, number=2,)


class ConversationTurn(proto.Message):
    r"""One interaction between a human and virtual agent. The human
    provides some input and the virtual agent provides a response.

    Attributes:
        user_input (google.cloud.dialogflowcx_v3.types.ConversationTurn.UserInput):
            The user input.
        virtual_agent_output (google.cloud.dialogflowcx_v3.types.ConversationTurn.VirtualAgentOutput):
            The virtual agent output.
    """

    class UserInput(proto.Message):
        r"""The input from the human user.
        Attributes:
            input (google.cloud.dialogflowcx_v3.types.QueryInput):
                Supports [text
                input][google.cloud.dialogflow.cx.v3.QueryInput.text],
                [event
                input][google.cloud.dialogflow.cx.v3.QueryInput.event],
                [dtmf input][google.cloud.dialogflow.cx.v3.QueryInput.dtmf]
                in the test case.
            injected_parameters (google.protobuf.struct_pb2.Struct):
                Parameters that need to be injected into the
                conversation during intent detection.
            is_webhook_enabled (bool):
                If webhooks should be allowed to trigger in
                response to the user utterance. Often if
                parameters are injected, webhooks should not be
                enabled.
            enable_sentiment_analysis (bool):
                Whether sentiment analysis is enabled.
        """

        input = proto.Field(proto.MESSAGE, number=5, message=session.QueryInput,)
        injected_parameters = proto.Field(
            proto.MESSAGE, number=2, message=struct_pb2.Struct,
        )
        is_webhook_enabled = proto.Field(proto.BOOL, number=3,)
        enable_sentiment_analysis = proto.Field(proto.BOOL, number=7,)

    class VirtualAgentOutput(proto.Message):
        r"""The output from the virtual agent.
        Attributes:
            session_parameters (google.protobuf.struct_pb2.Struct):
                The session parameters available to the bot
                at this point.
            differences (Sequence[google.cloud.dialogflowcx_v3.types.TestRunDifference]):
                Output only. If this is part of a [result conversation
                turn][TestCaseResult.conversation_turns], the list of
                differences between the original run and the replay for this
                output, if any.
            diagnostic_info (google.protobuf.struct_pb2.Struct):
                Required. Input only. The diagnostic
                [info][Session.DetectIntentResponse.QueryResult.diagnostic_info]
                output for the turn.
            triggered_intent (google.cloud.dialogflowcx_v3.types.Intent):
                The [Intent][google.cloud.dialogflow.cx.v3.Intent] that
                triggered the response. Only name and displayName will be
                set.
            current_page (google.cloud.dialogflowcx_v3.types.Page):
                The [Page][google.cloud.dialogflow.cx.v3.Page] on which the
                utterance was spoken. Only name and displayName will be set.
            text_responses (Sequence[google.cloud.dialogflowcx_v3.types.ResponseMessage.Text]):
                The
                [text][google.cloud.dialogflow.cx.v3.ResponseMessage.Text]
                responses from the agent for the turn.
            status (google.rpc.status_pb2.Status):
                Response error from the agent in the test
                result. If set, other output is empty.
        """

        session_parameters = proto.Field(
            proto.MESSAGE, number=4, message=struct_pb2.Struct,
        )
        differences = proto.RepeatedField(
            proto.MESSAGE, number=5, message="TestRunDifference",
        )
        diagnostic_info = proto.Field(
            proto.MESSAGE, number=6, message=struct_pb2.Struct,
        )
        triggered_intent = proto.Field(
            proto.MESSAGE, number=7, message=gcdc_intent.Intent,
        )
        current_page = proto.Field(proto.MESSAGE, number=8, message=gcdc_page.Page,)
        text_responses = proto.RepeatedField(
            proto.MESSAGE, number=9, message=response_message.ResponseMessage.Text,
        )
        status = proto.Field(proto.MESSAGE, number=10, message=status_pb2.Status,)

    user_input = proto.Field(proto.MESSAGE, number=1, message=UserInput,)
    virtual_agent_output = proto.Field(
        proto.MESSAGE, number=2, message=VirtualAgentOutput,
    )


class TestRunDifference(proto.Message):
    r"""The description of differences between original and replayed
    agent output.

    Attributes:
        type_ (google.cloud.dialogflowcx_v3.types.TestRunDifference.DiffType):
            The type of diff.
        description (str):
            A description of the diff, showing the actual
            output vs expected output.
    """

    class DiffType(proto.Enum):
        r"""What part of the message replay differs from the test case."""
        DIFF_TYPE_UNSPECIFIED = 0
        INTENT = 1
        PAGE = 2
        PARAMETERS = 3
        UTTERANCE = 4

    type_ = proto.Field(proto.ENUM, number=1, enum=DiffType,)
    description = proto.Field(proto.STRING, number=2,)


class TransitionCoverage(proto.Message):
    r"""Transition coverage represents the percentage of all possible
    page transitions (page-level transition routes and event
    handlers, excluding transition route groups) present within any
    of a parent's test cases.

    Attributes:
        transitions (Sequence[google.cloud.dialogflowcx_v3.types.TransitionCoverage.Transition]):
            The list of Transitions present in the agent.
        coverage_score (float):
            The percent of transitions in the agent that
            are covered.
    """

    class TransitionNode(proto.Message):
        r"""The source or target of a transition.
        Attributes:
            page (google.cloud.dialogflowcx_v3.types.Page):
                Indicates a transition to a
                [Page][google.cloud.dialogflow.cx.v3.Page]. Only some fields
                such as name and displayname will be set.
            flow (google.cloud.dialogflowcx_v3.types.Flow):
                Indicates a transition to a
                [Flow][google.cloud.dialogflow.cx.v3.Flow]. Only some fields
                such as name and displayname will be set.
        """

        page = proto.Field(
            proto.MESSAGE, number=1, oneof="kind", message=gcdc_page.Page,
        )
        flow = proto.Field(
            proto.MESSAGE, number=2, oneof="kind", message=gcdc_flow.Flow,
        )

    class Transition(proto.Message):
        r"""A transition in a page.
        Attributes:
            source (google.cloud.dialogflowcx_v3.types.TransitionCoverage.TransitionNode):
                The start node of a transition.
            index (int):
                The index of a transition in the transition
                list. Starting from 0.
            target (google.cloud.dialogflowcx_v3.types.TransitionCoverage.TransitionNode):
                The end node of a transition.
            covered (bool):
                Whether or not the transition is covered by
                at least one of the agent's test cases.
            transition_route (google.cloud.dialogflowcx_v3.types.TransitionRoute):
                Intent route or condition route.
            event_handler (google.cloud.dialogflowcx_v3.types.EventHandler):
                Event handler.
        """

        source = proto.Field(
            proto.MESSAGE, number=1, message="TransitionCoverage.TransitionNode",
        )
        index = proto.Field(proto.INT32, number=4,)
        target = proto.Field(
            proto.MESSAGE, number=2, message="TransitionCoverage.TransitionNode",
        )
        covered = proto.Field(proto.BOOL, number=3,)
        transition_route = proto.Field(
            proto.MESSAGE, number=5, oneof="detail", message=gcdc_page.TransitionRoute,
        )
        event_handler = proto.Field(
            proto.MESSAGE, number=6, oneof="detail", message=gcdc_page.EventHandler,
        )

    transitions = proto.RepeatedField(proto.MESSAGE, number=1, message=Transition,)
    coverage_score = proto.Field(proto.FLOAT, number=2,)


class TransitionRouteGroupCoverage(proto.Message):
    r"""Transition route group coverage represents the percentage of
    all possible transition routes present within any of a parent's
    test cases. The results are grouped by the transition route
    group.

    Attributes:
        coverages (Sequence[google.cloud.dialogflowcx_v3.types.TransitionRouteGroupCoverage.Coverage]):
            Transition route group coverages.
        coverage_score (float):
            The percent of transition routes in all the
            transition route groups that are covered.
    """

    class Coverage(proto.Message):
        r"""Coverage result message for one transition route group.
        Attributes:
            route_group (google.cloud.dialogflowcx_v3.types.TransitionRouteGroup):
                Transition route group metadata. Only name
                and displayName will be set.
            transitions (Sequence[google.cloud.dialogflowcx_v3.types.TransitionRouteGroupCoverage.Coverage.Transition]):
                The list of transition routes and coverage in
                the transition route group.
            coverage_score (float):
                The percent of transition routes in the
                transition route group that are covered.
        """

        class Transition(proto.Message):
            r"""A transition coverage in a transition route group.
            Attributes:
                transition_route (google.cloud.dialogflowcx_v3.types.TransitionRoute):
                    Intent route or condition route.
                covered (bool):
                    Whether or not the transition route is
                    covered by at least one of the agent's test
                    cases.
            """

            transition_route = proto.Field(
                proto.MESSAGE, number=1, message=gcdc_page.TransitionRoute,
            )
            covered = proto.Field(proto.BOOL, number=2,)

        route_group = proto.Field(
            proto.MESSAGE,
            number=1,
            message=transition_route_group.TransitionRouteGroup,
        )
        transitions = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="TransitionRouteGroupCoverage.Coverage.Transition",
        )
        coverage_score = proto.Field(proto.FLOAT, number=3,)

    coverages = proto.RepeatedField(proto.MESSAGE, number=1, message=Coverage,)
    coverage_score = proto.Field(proto.FLOAT, number=2,)


class IntentCoverage(proto.Message):
    r"""Intent coverage represents the percentage of all possible
    intents in the agent that are triggered in any of a parent's
    test cases.

    Attributes:
        intents (Sequence[google.cloud.dialogflowcx_v3.types.IntentCoverage.Intent]):
            The list of Intents present in the agent
        coverage_score (float):
            The percent of intents in the agent that are
            covered.
    """

    class Intent(proto.Message):
        r"""The agent's intent.
        Attributes:
            intent (str):
                The intent full resource name
            covered (bool):
                Whether or not the intent is covered by at
                least one of the agent's test cases.
        """

        intent = proto.Field(proto.STRING, number=1,)
        covered = proto.Field(proto.BOOL, number=2,)

    intents = proto.RepeatedField(proto.MESSAGE, number=1, message=Intent,)
    coverage_score = proto.Field(proto.FLOAT, number=2,)


class CalculateCoverageRequest(proto.Message):
    r"""The request message for
    [TestCases.CalculateCoverage][google.cloud.dialogflow.cx.v3.TestCases.CalculateCoverage].

    Attributes:
        agent (str):
            Required. The agent to calculate coverage for. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        type_ (google.cloud.dialogflowcx_v3.types.CalculateCoverageRequest.CoverageType):
            Required. The type of coverage requested.
    """

    class CoverageType(proto.Enum):
        r"""The type of coverage score requested."""
        COVERAGE_TYPE_UNSPECIFIED = 0
        INTENT = 1
        PAGE_TRANSITION = 2
        TRANSITION_ROUTE_GROUP = 3

    agent = proto.Field(proto.STRING, number=3,)
    type_ = proto.Field(proto.ENUM, number=2, enum=CoverageType,)


class CalculateCoverageResponse(proto.Message):
    r"""The response message for
    [TestCases.CalculateCoverage][google.cloud.dialogflow.cx.v3.TestCases.CalculateCoverage].

    Attributes:
        agent (str):
            The agent to calculate coverage for. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        intent_coverage (google.cloud.dialogflowcx_v3.types.IntentCoverage):
            Intent coverage.
        transition_coverage (google.cloud.dialogflowcx_v3.types.TransitionCoverage):
            Transition (excluding transition route
            groups) coverage.
        route_group_coverage (google.cloud.dialogflowcx_v3.types.TransitionRouteGroupCoverage):
            Transition route group coverage.
    """

    agent = proto.Field(proto.STRING, number=5,)
    intent_coverage = proto.Field(
        proto.MESSAGE, number=2, oneof="coverage_type", message="IntentCoverage",
    )
    transition_coverage = proto.Field(
        proto.MESSAGE, number=4, oneof="coverage_type", message="TransitionCoverage",
    )
    route_group_coverage = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="coverage_type",
        message="TransitionRouteGroupCoverage",
    )


class ListTestCasesRequest(proto.Message):
    r"""The request message for
    [TestCases.ListTestCases][google.cloud.dialogflow.cx.v3.TestCases.ListTestCases].

    Attributes:
        parent (str):
            Required. The agent to list all pages for. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        page_size (int):
            The maximum number of items to return in a
            single page. By default 20. Note that when
            TestCaseView = FULL, the maximum page size
            allowed is 20. When TestCaseView = BASIC, the
            maximum page size allowed is 500.
        page_token (str):
            The next_page_token value returned from a previous list
            request.
        view (google.cloud.dialogflowcx_v3.types.ListTestCasesRequest.TestCaseView):
            Specifies whether response should include all
            fields or just the metadata.
    """

    class TestCaseView(proto.Enum):
        r"""Specifies how much test case information to include in the
        response.
        """
        TEST_CASE_VIEW_UNSPECIFIED = 0
        BASIC = 1
        FULL = 2

    parent = proto.Field(proto.STRING, number=1,)
    page_size = proto.Field(proto.INT32, number=2,)
    page_token = proto.Field(proto.STRING, number=3,)
    view = proto.Field(proto.ENUM, number=4, enum=TestCaseView,)


class ListTestCasesResponse(proto.Message):
    r"""The response message for
    [TestCases.ListTestCases][google.cloud.dialogflow.cx.v3.TestCases.ListTestCases].

    Attributes:
        test_cases (Sequence[google.cloud.dialogflowcx_v3.types.TestCase]):
            The list of test cases. There will be a maximum number of
            items returned based on the page_size field in the request.
        next_page_token (str):
            Token to retrieve the next page of results,
            or empty if there are no more results in the
            list.
    """

    @property
    def raw_page(self):
        return self

    test_cases = proto.RepeatedField(proto.MESSAGE, number=1, message="TestCase",)
    next_page_token = proto.Field(proto.STRING, number=2,)


class BatchDeleteTestCasesRequest(proto.Message):
    r"""The request message for
    [TestCases.BatchDeleteTestCases][google.cloud.dialogflow.cx.v3.TestCases.BatchDeleteTestCases].

    Attributes:
        parent (str):
            Required. The agent to delete test cases from. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        names (Sequence[str]):
            Required. Format of test case names:
            ``projects/<Project ID>/locations/ <Location ID>/agents/<AgentID>/testCases/<TestCase ID>``.
    """

    parent = proto.Field(proto.STRING, number=1,)
    names = proto.RepeatedField(proto.STRING, number=3,)


class CreateTestCaseRequest(proto.Message):
    r"""The request message for
    [TestCases.CreateTestCase][google.cloud.dialogflow.cx.v3.TestCases.CreateTestCase].

    Attributes:
        parent (str):
            Required. The agent to create the test case for. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        test_case (google.cloud.dialogflowcx_v3.types.TestCase):
            Required. The test case to create.
    """

    parent = proto.Field(proto.STRING, number=1,)
    test_case = proto.Field(proto.MESSAGE, number=2, message="TestCase",)


class UpdateTestCaseRequest(proto.Message):
    r"""The request message for
    [TestCases.UpdateTestCase][google.cloud.dialogflow.cx.v3.TestCases.UpdateTestCase].

    Attributes:
        test_case (google.cloud.dialogflowcx_v3.types.TestCase):
            Required. The test case to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The mask to specify which fields should be
            updated. The
            [``creationTime``][google.cloud.dialogflow.cx.v3.TestCase.creation_time]
            and
            [``lastTestResult``][google.cloud.dialogflow.cx.v3.TestCase.last_test_result]
            cannot be updated.
    """

    test_case = proto.Field(proto.MESSAGE, number=1, message="TestCase",)
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class GetTestCaseRequest(proto.Message):
    r"""The request message for
    [TestCases.GetTestCase][google.cloud.dialogflow.cx.v3.TestCases.GetTestCase].

    Attributes:
        name (str):
            Required. The name of the testcase. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/testCases/<TestCase ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


class RunTestCaseRequest(proto.Message):
    r"""The request message for
    [TestCases.RunTestCase][google.cloud.dialogflow.cx.v3.TestCases.RunTestCase].

    Attributes:
        name (str):
            Required. Format of test case name to run:
            ``projects/<Project ID>/locations/ <Location ID>/agents/<AgentID>/testCases/<TestCase ID>``.
        environment (str):
            Optional. Environment name. If not set, draft environment is
            assumed. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)
    environment = proto.Field(proto.STRING, number=2,)


class RunTestCaseResponse(proto.Message):
    r"""The response message for
    [TestCases.RunTestCase][google.cloud.dialogflow.cx.v3.TestCases.RunTestCase].

    Attributes:
        result (google.cloud.dialogflowcx_v3.types.TestCaseResult):
            The result.
    """

    result = proto.Field(proto.MESSAGE, number=2, message="TestCaseResult",)


class RunTestCaseMetadata(proto.Message):
    r"""Metadata returned for the
    [TestCases.RunTestCase][google.cloud.dialogflow.cx.v3.TestCases.RunTestCase]
    long running operation.
        """


class BatchRunTestCasesRequest(proto.Message):
    r"""The request message for
    [TestCases.BatchRunTestCases][google.cloud.dialogflow.cx.v3.TestCases.BatchRunTestCases].

    Attributes:
        parent (str):
            Required. Agent name. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/ <AgentID>``.
        environment (str):
            Optional. If not set, draft environment is assumed. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>``.
        test_cases (Sequence[str]):
            Required. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/testCases/<TestCase ID>``.
    """

    parent = proto.Field(proto.STRING, number=1,)
    environment = proto.Field(proto.STRING, number=2,)
    test_cases = proto.RepeatedField(proto.STRING, number=3,)


class BatchRunTestCasesResponse(proto.Message):
    r"""The response message for
    [TestCases.BatchRunTestCases][google.cloud.dialogflow.cx.v3.TestCases.BatchRunTestCases].

    Attributes:
        results (Sequence[google.cloud.dialogflowcx_v3.types.TestCaseResult]):
            The test case results. The detailed [conversation
            turns][google.cloud.dialogflow.cx.v3.TestCaseResult.conversation_turns]
            are empty in this response.
    """

    results = proto.RepeatedField(proto.MESSAGE, number=1, message="TestCaseResult",)


class BatchRunTestCasesMetadata(proto.Message):
    r"""Metadata returned for the
    [TestCases.BatchRunTestCases][google.cloud.dialogflow.cx.v3.TestCases.BatchRunTestCases]
    long running operation.

    Attributes:
        errors (Sequence[google.cloud.dialogflowcx_v3.types.TestError]):
            The test errors.
    """

    errors = proto.RepeatedField(proto.MESSAGE, number=1, message="TestError",)


class TestError(proto.Message):
    r"""Error info for running a test.
    Attributes:
        test_case (str):
            The test case resource name.
        status (google.rpc.status_pb2.Status):
            The status associated with the test.
        test_time (google.protobuf.timestamp_pb2.Timestamp):
            The timestamp when the test was completed.
    """

    test_case = proto.Field(proto.STRING, number=1,)
    status = proto.Field(proto.MESSAGE, number=2, message=status_pb2.Status,)
    test_time = proto.Field(proto.MESSAGE, number=3, message=timestamp_pb2.Timestamp,)


class ImportTestCasesRequest(proto.Message):
    r"""The request message for
    [TestCases.ImportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ImportTestCases].

    Attributes:
        parent (str):
            Required. The agent to import test cases to. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        gcs_uri (str):
            The `Google Cloud
            Storage <https://cloud.google.com/storage/docs/>`__ URI to
            import test cases from. The format of this URI must be
            ``gs://<bucket-name>/<object-name>``.
        content (bytes):
            Uncompressed raw byte content for test cases.
    """

    parent = proto.Field(proto.STRING, number=1,)
    gcs_uri = proto.Field(proto.STRING, number=2, oneof="source",)
    content = proto.Field(proto.BYTES, number=3, oneof="source",)


class ImportTestCasesResponse(proto.Message):
    r"""The response message for
    [TestCases.ImportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ImportTestCases].

    Attributes:
        names (Sequence[str]):
            The unique identifiers of the new test cases. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/testCases/<TestCase ID>``.
    """

    names = proto.RepeatedField(proto.STRING, number=1,)


class ImportTestCasesMetadata(proto.Message):
    r"""Metadata returned for the
    [TestCases.ImportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ImportTestCases]
    long running operation.

    Attributes:
        errors (Sequence[google.cloud.dialogflowcx_v3.types.TestCaseError]):
            Errors for failed test cases.
    """

    errors = proto.RepeatedField(proto.MESSAGE, number=1, message="TestCaseError",)


class TestCaseError(proto.Message):
    r"""Error info for importing a test.
    Attributes:
        test_case (google.cloud.dialogflowcx_v3.types.TestCase):
            The test case.
        status (google.rpc.status_pb2.Status):
            The status associated with the test case.
    """

    test_case = proto.Field(proto.MESSAGE, number=1, message="TestCase",)
    status = proto.Field(proto.MESSAGE, number=2, message=status_pb2.Status,)


class ExportTestCasesRequest(proto.Message):
    r"""The request message for
    [TestCases.ExportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ExportTestCases].

    Attributes:
        parent (str):
            Required. The agent where to export test cases from. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        gcs_uri (str):
            The `Google Cloud
            Storage <https://cloud.google.com/storage/docs/>`__ URI to
            export the test cases to. The format of this URI must be
            ``gs://<bucket-name>/<object-name>``. If unspecified, the
            serialized test cases is returned inline.
        data_format (google.cloud.dialogflowcx_v3.types.ExportTestCasesRequest.DataFormat):
            The data format of the exported test cases. If not
            specified, ``BLOB`` is assumed.
        filter (str):
            The filter expression used to filter exported test cases,
            see `API Filtering <https://aip.dev/160>`__. The expression
            is case insensitive and supports the following syntax:

            name = [OR name = ] ...

            For example:

            -  "name = t1 OR name = t2" matches the test case with the
               exact resource name "t1" or "t2".
    """

    class DataFormat(proto.Enum):
        r"""Data format of the exported test cases."""
        DATA_FORMAT_UNSPECIFIED = 0
        BLOB = 1
        JSON = 2

    parent = proto.Field(proto.STRING, number=1,)
    gcs_uri = proto.Field(proto.STRING, number=2, oneof="destination",)
    data_format = proto.Field(proto.ENUM, number=3, enum=DataFormat,)
    filter = proto.Field(proto.STRING, number=4,)


class ExportTestCasesResponse(proto.Message):
    r"""The response message for
    [TestCases.ExportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ExportTestCases].

    Attributes:
        gcs_uri (str):
            The URI to a file containing the exported test cases. This
            field is populated only if ``gcs_uri`` is specified in
            [ExportTestCasesRequest][google.cloud.dialogflow.cx.v3.ExportTestCasesRequest].
        content (bytes):
            Uncompressed raw byte content for test cases.
    """

    gcs_uri = proto.Field(proto.STRING, number=1, oneof="destination",)
    content = proto.Field(proto.BYTES, number=2, oneof="destination",)


class ExportTestCasesMetadata(proto.Message):
    r"""Metadata returned for the
    [TestCases.ExportTestCases][google.cloud.dialogflow.cx.v3.TestCases.ExportTestCases]
    long running operation.
        """


class ListTestCaseResultsRequest(proto.Message):
    r"""The request message for
    [TestCases.ListTestCaseResults][google.cloud.dialogflow.cx.v3.TestCases.ListTestCaseResults].

    Attributes:
        parent (str):
            Required. The test case to list results for. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/ testCases/<TestCase ID>``.
            Specify a ``-`` as a wildcard for TestCase ID to list
            results across multiple test cases.
        page_size (int):
            The maximum number of items to return in a
            single page. By default 100 and at most 1000.
        page_token (str):
            The next_page_token value returned from a previous list
            request.
        filter (str):
            The filter expression used to filter test case results. See
            `API Filtering <https://aip.dev/160>`__.

            The expression is case insensitive. Only 'AND' is supported
            for logical operators. The supported syntax is listed below
            in detail:

             [AND ] ... [AND latest]

            The supported fields and operators are: field operator
            ``environment`` ``=``, ``IN`` (Use value ``draft`` for draft
            environment) ``test_time`` ``>``, ``<``

            ``latest`` only returns the latest test result in all
            results for each test case.

            Examples:

            -  "environment=draft AND latest" matches the latest test
               result for each test case in the draft environment.
            -  "environment IN (e1,e2)" matches any test case results
               with an environment resource name of either "e1" or "e2".
            -  "test_time > 1602540713" matches any test case results
               with test time later than a unix timestamp in seconds
               1602540713.
    """

    parent = proto.Field(proto.STRING, number=1,)
    page_size = proto.Field(proto.INT32, number=2,)
    page_token = proto.Field(proto.STRING, number=3,)
    filter = proto.Field(proto.STRING, number=4,)


class ListTestCaseResultsResponse(proto.Message):
    r"""The response message for
    [TestCases.ListTestCaseResults][google.cloud.dialogflow.cx.v3.TestCases.ListTestCaseResults].

    Attributes:
        test_case_results (Sequence[google.cloud.dialogflowcx_v3.types.TestCaseResult]):
            The list of test case results.
        next_page_token (str):
            Token to retrieve the next page of results,
            or empty if there are no more results in the
            list.
    """

    @property
    def raw_page(self):
        return self

    test_case_results = proto.RepeatedField(
        proto.MESSAGE, number=1, message="TestCaseResult",
    )
    next_page_token = proto.Field(proto.STRING, number=2,)


class GetTestCaseResultRequest(proto.Message):
    r"""The request message for
    [TestCases.GetTestCaseResult][google.cloud.dialogflow.cx.v3.TestCases.GetTestCaseResult].

    Attributes:
        name (str):
            Required. The name of the testcase. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/testCases/<TestCase ID>/results/<TestCaseResult ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.dialogflowcx_v3.types import flow
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.dialogflow.cx.v3",
    manifest={
        "SpeechToTextSettings",
        "Agent",
        "ListAgentsRequest",
        "ListAgentsResponse",
        "GetAgentRequest",
        "CreateAgentRequest",
        "UpdateAgentRequest",
        "DeleteAgentRequest",
        "ExportAgentRequest",
        "ExportAgentResponse",
        "RestoreAgentRequest",
        "ValidateAgentRequest",
        "GetAgentValidationResultRequest",
        "AgentValidationResult",
    },
)


class SpeechToTextSettings(proto.Message):
    r"""Settings related to speech recognition.
    Attributes:
        enable_speech_adaptation (bool):
            Whether to use speech adaptation for speech
            recognition.
    """

    enable_speech_adaptation = proto.Field(proto.BOOL, number=1,)


class Agent(proto.Message):
    r"""Agents are best described as Natural Language Understanding (NLU)
    modules that transform user requests into actionable data. You can
    include agents in your app, product, or service to determine user
    intent and respond to the user in a natural way.

    After you create an agent, you can add
    [Intents][google.cloud.dialogflow.cx.v3.Intent], [Entity
    Types][google.cloud.dialogflow.cx.v3.EntityType],
    [Flows][google.cloud.dialogflow.cx.v3.Flow],
    [Fulfillments][google.cloud.dialogflow.cx.v3.Fulfillment],
    [Webhooks][google.cloud.dialogflow.cx.v3.Webhook], and so on to
    manage the conversation flows..

    Attributes:
        name (str):
            The unique identifier of the agent. Required for the
            [Agents.UpdateAgent][google.cloud.dialogflow.cx.v3.Agents.UpdateAgent]
            method.
            [Agents.CreateAgent][google.cloud.dialogflow.cx.v3.Agents.CreateAgent]
            populates the name automatically. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        display_name (str):
            Required. The human-readable name of the
            agent, unique within the location.
        default_language_code (str):
            Required. Immutable. The default language of the agent as a
            language tag. See `Language
            Support <https://cloud.google.com/dialogflow/cx/docs/reference/language>`__
            for a list of the currently supported language codes. This
            field cannot be set by the
            [Agents.UpdateAgent][google.cloud.dialogflow.cx.v3.Agents.UpdateAgent]
            method.
        supported_language_codes (Sequence[str]):
            The list of all languages supported by the agent (except for
            the ``default_language_code``).
        time_zone (str):
            Required. The time zone of the agent from the `time zone
            database <https://www.iana.org/time-zones>`__, e.g.,
            America/New_York, Europe/Paris.
        description (str):
            The description of the agent. The maximum
            length is 500 characters. If exceeded, the
            request is rejected.
        avatar_uri (str):
            The URI of the agent's avatar. Avatars are used throughout
            the Dialogflow console and in the self-hosted `Web
            Demo <https://cloud.google.com/dialogflow/docs/integrations/web-demo>`__
            integration.
        speech_to_text_settings (google.cloud.dialogflowcx_v3.types.SpeechToTextSettings):
            Speech recognition related settings.
        start_flow (str):
            Immutable. Name of the start flow in this agent. A start
            flow will be automatically created when the agent is
            created, and can only be deleted by deleting the agent.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/flows/<Flow ID>``.
        security_settings (str):
            Name of the
            [SecuritySettings][google.cloud.dialogflow.cx.v3.SecuritySettings]
            reference for the agent. Format:
            ``projects/<Project ID>/locations/<Location ID>/securitySettings/<Security Settings ID>``.
        enable_stackdriver_logging (bool):
            Indicates if stackdriver logging is enabled
            for the agent.
        enable_spell_correction (bool):
            Indicates if automatic spell correction is
            enabled in detect intent requests.
    """

    name = proto.Field(proto.STRING, number=1,)
    display_name = proto.Field(proto.STRING, number=2,)
    default_language_code = proto.Field(proto.STRING, number=3,)
    supported_language_codes = proto.RepeatedField(proto.STRING, number=4,)
    time_zone = proto.Field(proto.STRING, number=5,)
    description = proto.Field(proto.STRING, number=6,)
    avatar_uri = proto.Field(proto.STRING, number=7,)
    speech_to_text_settings = proto.Field(
        proto.MESSAGE, number=13, message="SpeechToTextSettings",
    )
    start_flow = proto.Field(proto.STRING, number=16,)
    security_settings = proto.Field(proto.STRING, number=17,)
    enable_stackdriver_logging = proto.Field(proto.BOOL, number=18,)
    enable_spell_correction = proto.Field(proto.BOOL, number=20,)


class ListAgentsRequest(proto.Message):
    r"""The request message for
    [Agents.ListAgents][google.cloud.dialogflow.cx.v3.Agents.ListAgents].

    Attributes:
        parent (str):
            Required. The location to list all agents for. Format:
            ``projects/<Project ID>/locations/<Location ID>``.
        page_size (int):
            The maximum number of items to return in a
            single page. By default 100 and at most 1000.
        page_token (str):
            The next_page_token value returned from a previous list
            request.
    """

    parent = proto.Field(proto.STRING, number=1,)
    page_size = proto.Field(proto.INT32, number=2,)
    page_token = proto.Field(proto.STRING, number=3,)


class ListAgentsResponse(proto.Message):
    r"""The response message for
    [Agents.ListAgents][google.cloud.dialogflow.cx.v3.Agents.ListAgents].

    Attributes:
        agents (Sequence[google.cloud.dialogflowcx_v3.types.Agent]):
            The list of agents. There will be a maximum number of items
            returned based on the page_size field in the request.
        next_page_token (str):
            Token to retrieve the next page of results,
            or empty if there are no more results in the
            list.
    """

    @property
    def raw_page(self):
        return self

    agents = proto.RepeatedField(proto.MESSAGE, number=1, message="Agent",)
    next_page_token = proto.Field(proto.STRING, number=2,)


class GetAgentRequest(proto.Message):
    r"""The request message for
    [Agents.GetAgent][google.cloud.dialogflow.cx.v3.Agents.GetAgent].

    Attributes:
        name (str):
            Required. The name of the agent. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


class CreateAgentRequest(proto.Message):
    r"""The request message for
    [Agents.CreateAgent][google.cloud.dialogflow.cx.v3.Agents.CreateAgent].

    Attributes:
        parent (str):
            Required. The location to create a agent for. Format:
            ``projects/<Project ID>/locations/<Location ID>``.
        agent (google.cloud.dialogflowcx_v3.types.Agent):
            Required. The agent to create.
    """

    parent = proto.Field(proto.STRING, number=1,)
    agent = proto.Field(proto.MESSAGE, number=2, message="Agent",)


class UpdateAgentRequest(proto.Message):
    r"""The request message for
    [Agents.UpdateAgent][google.cloud.dialogflow.cx.v3.Agents.UpdateAgent].

    Attributes:
        agent (google.cloud.dialogflowcx_v3.types.Agent):
            Required. The agent to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            The mask to control which fields get updated.
            If the mask is not present, all fields will be
            updated.
    """

    agent = proto.Field(proto.MESSAGE, number=1, message="Agent",)
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class DeleteAgentRequest(proto.Message):
    r"""The request message for
    [Agents.DeleteAgent][google.cloud.dialogflow.cx.v3.Agents.DeleteAgent].

    Attributes:
        name (str):
            Required. The name of the agent to delete. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


class ExportAgentRequest(proto.Message):
    r"""The request message for
    [Agents.ExportAgent][google.cloud.dialogflow.cx.v3.Agents.ExportAgent].

    Attributes:
        name (str):
            Required. The name of the agent to export. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        agent_uri (str):
            Optional. The `Google Cloud
            Storage <https://cloud.google.com/storage/docs/>`__ URI to
            export the agent to. The format of this URI must be
            ``gs://<bucket-name>/<object-name>``. If left unspecified,
            the serialized agent is returned inline.
        environment (str):
            Optional. Environment name. If not set, draft environment is
            assumed. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)
    agent_uri = proto.Field(proto.STRING, number=2,)
    environment = proto.Field(proto.STRING, number=5,)


class ExportAgentResponse(proto.Message):
    r"""The response message for
    [Agents.ExportAgent][google.cloud.dialogflow.cx.v3.Agents.ExportAgent].

    Attributes:
        agent_uri (str):
            The URI to a file containing the exported agent. This field
            is populated only if ``agent_uri`` is specified in
            [ExportAgentRequest][google.cloud.dialogflow.cx.v3.ExportAgentRequest].
        agent_content (bytes):
            Uncompressed raw byte content for agent.
    """

    agent_uri = proto.Field(proto.STRING, number=1, oneof="agent",)
    agent_content = proto.Field(proto.BYTES, number=2, oneof="agent",)


class RestoreAgentRequest(proto.Message):
    r"""The request message for
    [Agents.RestoreAgent][google.cloud.dialogflow.cx.v3.Agents.RestoreAgent].

    Attributes:
        name (str):
            Required. The name of the agent to restore into. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        agent_uri (str):
            The `Google Cloud
            Storage <https://cloud.google.com/storage/docs/>`__ URI to
            restore agent from. The format of this URI must be
            ``gs://<bucket-name>/<object-name>``.
        agent_content (bytes):
            Uncompressed raw byte content for agent.
        restore_option (google.cloud.dialogflowcx_v3.types.RestoreAgentRequest.RestoreOption):
            Agent restore mode. If not specified, ``KEEP`` is assumed.
    """

    class RestoreOption(proto.Enum):
        r"""Restore option."""
        RESTORE_OPTION_UNSPECIFIED = 0
        KEEP = 1
        FALLBACK = 2

    name = proto.Field(proto.STRING, number=1,)
    agent_uri = proto.Field(proto.STRING, number=2, oneof="agent",)
    agent_content = proto.Field(proto.BYTES, number=3, oneof="agent",)
    restore_option = proto.Field(proto.ENUM, number=5, enum=RestoreOption,)


class ValidateAgentRequest(proto.Message):
    r"""The request message for
    [Agents.ValidateAgent][google.cloud.dialogflow.cx.v3.Agents.ValidateAgent].

    Attributes:
        name (str):
            Required. The agent to validate. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>``.
        language_code (str):
            If not specified, the agent's default
            language is used.
    """

    name = proto.Field(proto.STRING, number=1,)
    language_code = proto.Field(proto.STRING, number=2,)


class GetAgentValidationResultRequest(proto.Message):
    r"""The request message for
    [Agents.GetAgentValidationResult][google.cloud.dialogflow.cx.v3.Agents.GetAgentValidationResult].

    Attributes:
        name (str):
            Required. The agent name. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/validationResult``.
        language_code (str):
            If not specified, the agent's default
            language is used.
    """

    name = proto.Field(proto.STRING, number=1,)
    language_code = proto.Field(proto.STRING, number=2,)


class AgentValidationResult(proto.Message):
    r"""The response message for
    [Agents.GetAgentValidationResult][google.cloud.dialogflow.cx.v3.Agents.GetAgentValidationResult].

    Attributes:
        name (str):
            The unique identifier of the agent validation result.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/validationResult``.
        flow_validation_results (Sequence[google.cloud.dialogflowcx_v3.types.FlowValidationResult]):
            Contains all flow validation results.
    """

    name = proto.Field(proto.STRING, number=1,)
    flow_validation_results = proto.RepeatedField(
        proto.MESSAGE, number=2, message=flow.FlowValidationResult,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.dialogflowcx_v3.types import audio_config
from google.cloud.dialogflowcx_v3.types import intent as gcdc_intent
from google.cloud.dialogflowcx_v3.types import page
from google.cloud.dialogflowcx_v3.types import response_message
from google.cloud.dialogflowcx_v3.types import session_entity_type
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import struct_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
from google.type import latlng_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.dialogflow.cx.v3",
    manifest={
        "DetectIntentRequest",
        "DetectIntentResponse",
        "StreamingDetectIntentRequest",
        "StreamingDetectIntentResponse",
        "StreamingRecognitionResult",
        "QueryParameters",
        "QueryInput",
        "QueryResult",
        "TextInput",
        "IntentInput",
        "AudioInput",
        "EventInput",
        "DtmfInput",
        "Match",
        "MatchIntentRequest",
        "MatchIntentResponse",
        "FulfillIntentRequest",
        "FulfillIntentResponse",
        "SentimentAnalysisResult",
    },
)


class DetectIntentRequest(proto.Message):
    r"""The request to detect user's intent.
    Attributes:
        session (str):
            Required. The name of the session this query is sent to.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/sessions/<Session ID>``
            or
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>``.
            If ``Environment ID`` is not specified, we assume default
            'draft' environment. It's up to the API caller to choose an
            appropriate ``Session ID``. It can be a random number or
            some type of session identifiers (preferably hashed). The
            length of the ``Session ID`` must not exceed 36 characters.

            For more information, see the `sessions
            guide <https://cloud.google.com/dialogflow/cx/docs/concept/session>`__.

            Note: Always use agent versions for production traffic. See
            `Versions and
            environments <https://cloud.google.com/dialogflow/cx/docs/concept/version>`__.
        query_params (google.cloud.dialogflowcx_v3.types.QueryParameters):
            The parameters of this query.
        query_input (google.cloud.dialogflowcx_v3.types.QueryInput):
            Required. The input specification.
        output_audio_config (google.cloud.dialogflowcx_v3.types.OutputAudioConfig):
            Instructs the speech synthesizer how to
            generate the output audio.
    """

    session = proto.Field(proto.STRING, number=1,)
    query_params = proto.Field(proto.MESSAGE, number=2, message="QueryParameters",)
    query_input = proto.Field(proto.MESSAGE, number=3, message="QueryInput",)
    output_audio_config = proto.Field(
        proto.MESSAGE, number=4, message=audio_config.OutputAudioConfig,
    )


class DetectIntentResponse(proto.Message):
    r"""The message returned from the DetectIntent method.
    Attributes:
        response_id (str):
            Output only. The unique identifier of the
            response. It can be used to locate a response in
            the training example set or for reporting
            issues.
        query_result (google.cloud.dialogflowcx_v3.types.QueryResult):
            The result of the conversational query.
        output_audio (bytes):
            The audio data bytes encoded as specified in the request.
            Note: The output audio is generated based on the values of
            default platform text responses found in the
            [``query_result.response_messages``][google.cloud.dialogflow.cx.v3.QueryResult.response_messages]
            field. If multiple default text responses exist, they will
            be concatenated when generating audio. If no default
            platform text responses exist, the generated audio content
            will be empty.

            In some scenarios, multiple output audio fields may be
            present in the response structure. In these cases, only the
            top-most-level audio output has content.
        output_audio_config (google.cloud.dialogflowcx_v3.types.OutputAudioConfig):
            The config used by the speech synthesizer to
            generate the output audio.
        response_type (google.cloud.dialogflowcx_v3.types.DetectIntentResponse.ResponseType):
            Response type.
        allow_cancellation (bool):
            Indicates whether the partial response can be
            cancelled when a later response arrives. e.g. if
            the agent specified some music as partial
            response, it can be cancelled.
    """

    class ResponseType(proto.Enum):
        r"""Represents different DetectIntentResponse types."""
        RESPONSE_TYPE_UNSPECIFIED = 0
        PARTIAL = 1
        FINAL = 2

    response_id = proto.Field(proto.STRING, number=1,)
    query_result = proto.Field(proto.MESSAGE, number=2, message="QueryResult",)
    output_audio = proto.Field(proto.BYTES, number=4,)
    output_audio_config = proto.Field(
        proto.MESSAGE, number=5, message=audio_config.OutputAudioConfig,
    )
    response_type = proto.Field(proto.ENUM, number=6, enum=ResponseType,)
    allow_cancellation = proto.Field(proto.BOOL, number=7,)


class StreamingDetectIntentRequest(proto.Message):
    r"""The top-level message sent by the client to the
    [Sessions.StreamingDetectIntent][google.cloud.dialogflow.cx.v3.Sessions.StreamingDetectIntent]
    method.

    Multiple request messages should be sent in order:

    1. The first message must contain
       [session][google.cloud.dialogflow.cx.v3.StreamingDetectIntentRequest.session],
       [query_input][google.cloud.dialogflow.cx.v3.StreamingDetectIntentRequest.query_input]
       plus optionally
       [query_params][google.cloud.dialogflow.cx.v3.StreamingDetectIntentRequest.query_params].
       If the client wants to receive an audio response, it should also
       contain
       [output_audio_config][google.cloud.dialogflow.cx.v3.StreamingDetectIntentRequest.output_audio_config].

    2. If
       [query_input][google.cloud.dialogflow.cx.v3.StreamingDetectIntentRequest.query_input]
       was set to
       [query_input.audio.config][google.cloud.dialogflow.cx.v3.AudioInput.config],
       all subsequent messages must contain
       [query_input.audio.audio][google.cloud.dialogflow.cx.v3.AudioInput.audio]
       to continue with Speech recognition. If you decide to rather
       detect an intent from text input after you already started Speech
       recognition, please send a message with
       [query_input.text][google.cloud.dialogflow.cx.v3.QueryInput.text].

       However, note that:

       -  Dialogflow will bill you for the audio duration so far.
       -  Dialogflow discards all Speech recognition results in favor of
          the input text.
       -  Dialogflow will use the language code from the first message.

    After you sent all input, you must half-close or abort the request
    stream.

    Attributes:
        session (str):
            The name of the session this query is sent to. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/sessions/<Session ID>``
            or
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>``.
            If ``Environment ID`` is not specified, we assume default
            'draft' environment. It's up to the API caller to choose an
            appropriate ``Session ID``. It can be a random number or
            some type of session identifiers (preferably hashed). The
            length of the ``Session ID`` must not exceed 36 characters.
            Note: session must be set in the first request.

            For more information, see the `sessions
            guide <https://cloud.google.com/dialogflow/cx/docs/concept/session>`__.

            Note: Always use agent versions for production traffic. See
            `Versions and
            environments <https://cloud.google.com/dialogflow/cx/docs/concept/version>`__.
        query_params (google.cloud.dialogflowcx_v3.types.QueryParameters):
            The parameters of this query.
        query_input (google.cloud.dialogflowcx_v3.types.QueryInput):
            Required. The input specification.
        output_audio_config (google.cloud.dialogflowcx_v3.types.OutputAudioConfig):
            Instructs the speech synthesizer how to
            generate the output audio.
        enable_partial_response (bool):
            Enable partial detect intent response. If this flag is not
            enabled, response stream still contains only one final
            ``DetectIntentResponse`` even if some ``Fulfillment``\ s in
            the agent have been configured to return partial responses.
    """

    session = proto.Field(proto.STRING, number=1,)
    query_params = proto.Field(proto.MESSAGE, number=2, message="QueryParameters",)
    query_input = proto.Field(proto.MESSAGE, number=3, message="QueryInput",)
    output_audio_config = proto.Field(
        proto.MESSAGE, number=4, message=audio_config.OutputAudioConfig,
    )
    enable_partial_response = proto.Field(proto.BOOL, number=5,)


class StreamingDetectIntentResponse(proto.Message):
    r"""The top-level message returned from the ``StreamingDetectIntent``
    method.

    Multiple response messages can be returned in order:

    1. If the input was set to streaming audio, the first one or more
       messages contain ``recognition_result``. Each
       ``recognition_result`` represents a more complete transcript of
       what the user said. The last ``recognition_result`` has
       ``is_final`` set to ``true``.

    2. If ``enable_partial_response`` is true, the following N messages
       (currently 1 <= N <= 4) contain ``detect_intent_response``. The
       first (N-1) ``detect_intent_response``\ s will have
       ``response_type`` set to ``PARTIAL``. The last
       ``detect_intent_response`` has ``response_type`` set to
       ``FINAL``. If ``response_type`` is false, response stream only
       contains the final ``detect_intent_response``.

    Attributes:
        recognition_result (google.cloud.dialogflowcx_v3.types.StreamingRecognitionResult):
            The result of speech recognition.
        detect_intent_response (google.cloud.dialogflowcx_v3.types.DetectIntentResponse):
            The response from detect intent.
    """

    recognition_result = proto.Field(
        proto.MESSAGE, number=1, oneof="response", message="StreamingRecognitionResult",
    )
    detect_intent_response = proto.Field(
        proto.MESSAGE, number=2, oneof="response", message="DetectIntentResponse",
    )


class StreamingRecognitionResult(proto.Message):
    r"""Contains a speech recognition result corresponding to a portion of
    the audio that is currently being processed or an indication that
    this is the end of the single requested utterance.

    Example:

    1. transcript: "tube"

    2. transcript: "to be a"

    3. transcript: "to be"

    4. transcript: "to be or not to be" is_final: true

    5. transcript: " that's"

    6. transcript: " that is"

    7. message_type: ``END_OF_SINGLE_UTTERANCE``

    8. transcript: " that is the question" is_final: true

    Only two of the responses contain final results (#4 and #8 indicated
    by ``is_final: true``). Concatenating these generates the full
    transcript: "to be or not to be that is the question".

    In each response we populate:

    -  for ``TRANSCRIPT``: ``transcript`` and possibly ``is_final``.

    -  for ``END_OF_SINGLE_UTTERANCE``: only ``message_type``.

    Attributes:
        message_type (google.cloud.dialogflowcx_v3.types.StreamingRecognitionResult.MessageType):
            Type of the result message.
        transcript (str):
            Transcript text representing the words that the user spoke.
            Populated if and only if ``message_type`` = ``TRANSCRIPT``.
        is_final (bool):
            If ``false``, the ``StreamingRecognitionResult`` represents
            an interim result that may change. If ``true``, the
            recognizer will not return any further hypotheses about this
            piece of the audio. May only be populated for
            ``message_type`` = ``TRANSCRIPT``.
        confidence (float):
            The Speech confidence between 0.0 and 1.0 for the current
            portion of audio. A higher number indicates an estimated
            greater likelihood that the recognized words are correct.
            The default of 0.0 is a sentinel value indicating that
            confidence was not set.

            This field is typically only provided if ``is_final`` is
            true and you should not rely on it being accurate or even
            set.
        stability (float):
            An estimate of the likelihood that the speech recognizer
            will not change its guess about this interim recognition
            result:

            -  If the value is unspecified or 0.0, Dialogflow didn't
               compute the stability. In particular, Dialogflow will
               only provide stability for ``TRANSCRIPT`` results with
               ``is_final = false``.
            -  Otherwise, the value is in (0.0, 1.0] where 0.0 means
               completely unstable and 1.0 means completely stable.
        speech_word_info (Sequence[google.cloud.dialogflowcx_v3.types.SpeechWordInfo]):
            Word-specific information for the words recognized by Speech
            in
            [transcript][google.cloud.dialogflow.cx.v3.StreamingRecognitionResult.transcript].
            Populated if and only if ``message_type`` = ``TRANSCRIPT``
            and [InputAudioConfig.enable_word_info] is set.
        speech_end_offset (google.protobuf.duration_pb2.Duration):
            Time offset of the end of this Speech recognition result
            relative to the beginning of the audio. Only populated for
            ``message_type`` = ``TRANSCRIPT``.
    """

    class MessageType(proto.Enum):
        r"""Type of the response message."""
        MESSAGE_TYPE_UNSPECIFIED = 0
        TRANSCRIPT = 1
        END_OF_SINGLE_UTTERANCE = 2

    message_type = proto.Field(proto.ENUM, number=1, enum=MessageType,)
    transcript = proto.Field(proto.STRING, number=2,)
    is_final = proto.Field(proto.BOOL, number=3,)
    confidence = proto.Field(proto.FLOAT, number=4,)
    stability = proto.Field(proto.FLOAT, number=6,)
    speech_word_info = proto.RepeatedField(
        proto.MESSAGE, number=7, message=audio_config.SpeechWordInfo,
    )
    speech_end_offset = proto.Field(
        proto.MESSAGE, number=8, message=duration_pb2.Duration,
    )


class QueryParameters(proto.Message):
    r"""Represents the parameters of a conversational query.
    Attributes:
        time_zone (str):
            The time zone of this conversational query from the `time
            zone database <https://www.iana.org/time-zones>`__, e.g.,
            America/New_York, Europe/Paris. If not provided, the time
            zone specified in the agent is used.
        geo_location (google.type.latlng_pb2.LatLng):
            The geo location of this conversational
            query.
        session_entity_types (Sequence[google.cloud.dialogflowcx_v3.types.SessionEntityType]):
            Additional session entity types to replace or
            extend developer entity types with. The entity
            synonyms apply to all languages and persist for
            the session of this query.
        payload (google.protobuf.struct_pb2.Struct):
            This field can be used to pass custom data into the webhook
            associated with the agent. Arbitrary JSON objects are
            supported. Some integrations that query a Dialogflow agent
            may provide additional information in the payload. In
            particular, for the Dialogflow Phone Gateway integration,
            this field has the form:

            ::

               {
                "telephony": {
                  "caller_id": "+18558363987"
                }
               }
        parameters (google.protobuf.struct_pb2.Struct):
            Additional parameters to be put into [session
            parameters][SessionInfo.parameters]. To remove a parameter
            from the session, clients should explicitly set the
            parameter value to null.

            You can reference the session parameters in the agent with
            the following format: $session.params.parameter-id.

            Depending on your protocol or client library language, this
            is a map, associative array, symbol table, dictionary, or
            JSON object composed of a collection of (MapKey, MapValue)
            pairs:

            -  MapKey type: string
            -  MapKey value: parameter name
            -  MapValue type:

               -  If parameter's entity type is a composite entity: map
               -  Else: depending on parameter value type, could be one
                  of string, number, boolean, null, list or map

            -  MapValue value:

               -  If parameter's entity type is a composite entity: map
                  from composite entity property names to property
                  values
               -  Else: parameter value
        current_page (str):
            The unique identifier of the
            [page][google.cloud.dialogflow.cx.v3.Page] to override the
            [current page][QueryResult.current_page] in the session.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/flows/<Flow ID>/pages/<Page ID>``.

            If ``current_page`` is specified, the previous state of the
            session will be ignored by Dialogflow, including the
            [previous page][QueryResult.current_page] and the [previous
            session parameters][QueryResult.parameters]. In most cases,
            [current_page][google.cloud.dialogflow.cx.v3.QueryParameters.current_page]
            and
            [parameters][google.cloud.dialogflow.cx.v3.QueryParameters.parameters]
            should be configured together to direct a session to a
            specific state.
        disable_webhook (bool):
            Whether to disable webhook calls for this
            request.
        analyze_query_text_sentiment (bool):
            Configures whether sentiment analysis should
            be performed. If not provided, sentiment
            analysis is not performed.
        webhook_headers (Sequence[google.cloud.dialogflowcx_v3.types.QueryParameters.WebhookHeadersEntry]):
            This field can be used to pass HTTP headers
            for a webhook call. These headers will be sent
            to webhook along with the headers that have been
            configured through Dialogflow web console. The
            headers defined within this field will overwrite
            the headers configured through Dialogflow
            console if there is a conflict. Header names are
            case-insensitive. Google's specified headers are
            not allowed. Including: "Host", "Content-
            Length", "Connection", "From", "User-Agent",
            "Accept-Encoding", "If-Modified-Since", "If-
            None-Match", "X-Forwarded-For", etc.
    """

    time_zone = proto.Field(proto.STRING, number=1,)
    geo_location = proto.Field(proto.MESSAGE, number=2, message=latlng_pb2.LatLng,)
    session_entity_types = proto.RepeatedField(
        proto.MESSAGE, number=3, message=session_entity_type.SessionEntityType,
    )
    payload = proto.Field(proto.MESSAGE, number=4, message=struct_pb2.Struct,)
    parameters = proto.Field(proto.MESSAGE, number=5, message=struct_pb2.Struct,)
    current_page = proto.Field(proto.STRING, number=6,)
    disable_webhook = proto.Field(proto.BOOL, number=7,)
    analyze_query_text_sentiment = proto.Field(proto.BOOL, number=8,)
    webhook_headers = proto.MapField(proto.STRING, proto.STRING, number=10,)


class QueryInput(proto.Message):
    r"""Represents the query input. It can contain one of:
    1.  A conversational query in the form of text.

    2.  An intent query that specifies which intent to trigger.
    3.  Natural language speech audio to be processed.

    4.  An event to be triggered.

    Attributes:
        text (google.cloud.dialogflowcx_v3.types.TextInput):
            The natural language text to be processed.
        intent (google.cloud.dialogflowcx_v3.types.IntentInput):
            The intent to be triggered.
        audio (google.cloud.dialogflowcx_v3.types.AudioInput):
            The natural language speech audio to be
            processed.
        event (google.cloud.dialogflowcx_v3.types.EventInput):
            The event to be triggered.
        dtmf (google.cloud.dialogflowcx_v3.types.DtmfInput):
            The DTMF event to be handled.
        language_code (str):
            Required. The language of the input. See `Language
            Support <https://cloud.google.com/dialogflow/cx/docs/reference/language>`__
            for a list of the currently supported language codes. Note
            that queries in the same session do not necessarily need to
            specify the same language.
    """

    text = proto.Field(proto.MESSAGE, number=2, oneof="input", message="TextInput",)
    intent = proto.Field(proto.MESSAGE, number=3, oneof="input", message="IntentInput",)
    audio = proto.Field(proto.MESSAGE, number=5, oneof="input", message="AudioInput",)
    event = proto.Field(proto.MESSAGE, number=6, oneof="input", message="EventInput",)
    dtmf = proto.Field(proto.MESSAGE, number=7, oneof="input", message="DtmfInput",)
    language_code = proto.Field(proto.STRING, number=4,)


class QueryResult(proto.Message):
    r"""Represents the result of a conversational query.
    Attributes:
        text (str):
            If [natural language
            text][google.cloud.dialogflow.cx.v3.TextInput] was provided
            as input, this field will contain a copy of the text.
        trigger_intent (str):
            If an [intent][google.cloud.dialogflow.cx.v3.IntentInput]
            was provided as input, this field will contain a copy of the
            intent identifier. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/intents/<Intent ID>``.
        transcript (str):
            If [natural language speech
            audio][google.cloud.dialogflow.cx.v3.AudioInput] was
            provided as input, this field will contain the transcript
            for the audio.
        trigger_event (str):
            If an [event][google.cloud.dialogflow.cx.v3.EventInput] was
            provided as input, this field will contain the name of the
            event.
        language_code (str):
            The language that was triggered during intent detection. See
            `Language
            Support <https://cloud.google.com/dialogflow/cx/docs/reference/language>`__
            for a list of the currently supported language codes.
        parameters (google.protobuf.struct_pb2.Struct):
            The collected [session
            parameters][google.cloud.dialogflow.cx.v3.SessionInfo.parameters].

            Depending on your protocol or client library language, this
            is a map, associative array, symbol table, dictionary, or
            JSON object composed of a collection of (MapKey, MapValue)
            pairs:

            -  MapKey type: string
            -  MapKey value: parameter name
            -  MapValue type:

               -  If parameter's entity type is a composite entity: map
               -  Else: depending on parameter value type, could be one
                  of string, number, boolean, null, list or map

            -  MapValue value:

               -  If parameter's entity type is a composite entity: map
                  from composite entity property names to property
                  values
               -  Else: parameter value
        response_messages (Sequence[google.cloud.dialogflowcx_v3.types.ResponseMessage]):
            The list of rich messages returned to the
            client. Responses vary from simple text messages
            to more sophisticated, structured payloads used
            to drive complex logic.
        webhook_statuses (Sequence[google.rpc.status_pb2.Status]):
            The list of webhook call status in the order
            of call sequence.
        webhook_payloads (Sequence[google.protobuf.struct_pb2.Struct]):
            The list of webhook payload in
            [WebhookResponse.payload][google.cloud.dialogflow.cx.v3.WebhookResponse.payload],
            in the order of call sequence. If some webhook call fails or
            doesn't return any payload, an empty ``Struct`` would be
            used instead.
        current_page (google.cloud.dialogflowcx_v3.types.Page):
            The current [Page][google.cloud.dialogflow.cx.v3.Page].
            Some, not all fields are filled in this message, including
            but not limited to ``name`` and ``display_name``.
        intent (google.cloud.dialogflowcx_v3.types.Intent):
            The [Intent][google.cloud.dialogflow.cx.v3.Intent] that
            matched the conversational query. Some, not all fields are
            filled in this message, including but not limited to:
            ``name`` and ``display_name``. This field is deprecated,
            please use
            [QueryResult.match][google.cloud.dialogflow.cx.v3.QueryResult.match]
            instead.
        intent_detection_confidence (float):
            The intent detection confidence. Values range from 0.0
            (completely uncertain) to 1.0 (completely certain). This
            value is for informational purpose only and is only used to
            help match the best intent within the classification
            threshold. This value may change for the same end-user
            expression at any time due to a model retraining or change
            in implementation. This field is deprecated, please use
            [QueryResult.match][google.cloud.dialogflow.cx.v3.QueryResult.match]
            instead.
        match (google.cloud.dialogflowcx_v3.types.Match):
            Intent match result, could be an intent or an
            event.
        diagnostic_info (google.protobuf.struct_pb2.Struct):
            The free-form diagnostic info. For example,
            this field could contain webhook call latency.
            The string keys of the Struct's fields map can
            change without notice.
        sentiment_analysis_result (google.cloud.dialogflowcx_v3.types.SentimentAnalysisResult):
            The sentiment analyss result, which depends on
            [``analyze_query_text_sentiment``]
            [google.cloud.dialogflow.cx.v3.QueryParameters.analyze_query_text_sentiment],
            specified in the request.
    """

    text = proto.Field(proto.STRING, number=1, oneof="query",)
    trigger_intent = proto.Field(proto.STRING, number=11, oneof="query",)
    transcript = proto.Field(proto.STRING, number=12, oneof="query",)
    trigger_event = proto.Field(proto.STRING, number=14, oneof="query",)
    language_code = proto.Field(proto.STRING, number=2,)
    parameters = proto.Field(proto.MESSAGE, number=3, message=struct_pb2.Struct,)
    response_messages = proto.RepeatedField(
        proto.MESSAGE, number=4, message=response_message.ResponseMessage,
    )
    webhook_statuses = proto.RepeatedField(
        proto.MESSAGE, number=13, message=status_pb2.Status,
    )
    webhook_payloads = proto.RepeatedField(
        proto.MESSAGE, number=6, message=struct_pb2.Struct,
    )
    current_page = proto.Field(proto.MESSAGE, number=7, message=page.Page,)
    intent = proto.Field(proto.MESSAGE, number=8, message=gcdc_intent.Intent,)
    intent_detection_confidence = proto.Field(proto.FLOAT, number=9,)
    match = proto.Field(proto.MESSAGE, number=15, message="Match",)
    diagnostic_info = proto.Field(proto.MESSAGE, number=10, message=struct_pb2.Struct,)
    sentiment_analysis_result = proto.Field(
        proto.MESSAGE, number=17, message="SentimentAnalysisResult",
    )


class TextInput(proto.Message):
    r"""Represents the natural language text to be processed.
    Attributes:
        text (str):
            Required. The UTF-8 encoded natural language
            text to be processed. Text length must not
            exceed 256 characters.
    """

    text = proto.Field(proto.STRING, number=1,)


class IntentInput(proto.Message):
    r"""Represents the intent to trigger programmatically rather than
    as a result of natural language processing.

    Attributes:
        intent (str):
            Required. The unique identifier of the intent. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/intents/<Intent ID>``.
    """

    intent = proto.Field(proto.STRING, number=1,)


class AudioInput(proto.Message):
    r"""Represents the natural speech audio to be processed.
    Attributes:
        config (google.cloud.dialogflowcx_v3.types.InputAudioConfig):
            Required. Instructs the speech recognizer how
            to process the speech audio.
        audio (bytes):
            The natural language speech audio to be processed. A single
            request can contain up to 1 minute of speech audio data. The
            [transcribed
            text][google.cloud.dialogflow.cx.v3.QueryResult.transcript]
            cannot contain more than 256 bytes.

            For non-streaming audio detect intent, both ``config`` and
            ``audio`` must be provided. For streaming audio detect
            intent, ``config`` must be provided in the first request and
            ``audio`` must be provided in all following requests.
    """

    config = proto.Field(
        proto.MESSAGE, number=1, message=audio_config.InputAudioConfig,
    )
    audio = proto.Field(proto.BYTES, number=2,)


class EventInput(proto.Message):
    r"""Represents the event to trigger.
    Attributes:
        event (str):
            Name of the event.
    """

    event = proto.Field(proto.STRING, number=1,)


class DtmfInput(proto.Message):
    r"""Represents the input for dtmf event.
    Attributes:
        digits (str):
            The dtmf digits.
        finish_digit (str):
            The finish digit (if any).
    """

    digits = proto.Field(proto.STRING, number=1,)
    finish_digit = proto.Field(proto.STRING, number=2,)


class Match(proto.Message):
    r"""Represents one match result of [MatchIntent][].
    Attributes:
        intent (google.cloud.dialogflowcx_v3.types.Intent):
            The [Intent][google.cloud.dialogflow.cx.v3.Intent] that
            matched the query. Some, not all fields are filled in this
            message, including but not limited to: ``name`` and
            ``display_name``. Only filled for
            [``INTENT``][google.cloud.dialogflow.cx.v3.Match.MatchType]
            match type.
        event (str):
            The event that matched the query. Only filled for
            [``EVENT``][google.cloud.dialogflow.cx.v3.Match.MatchType]
            match type.
        parameters (google.protobuf.struct_pb2.Struct):
            The collection of parameters extracted from
            the query.
            Depending on your protocol or client library
            language, this is a map, associative array,
            symbol table, dictionary, or JSON object
            composed of a collection of (MapKey, MapValue)
            pairs:
            -   MapKey type: string
            -   MapKey value: parameter name
            -   MapValue type:
                -   If parameter's entity type is a
            composite entity: map     -   Else: depending on
            parameter value type, could be one of string,
            number, boolean, null, list or map
            -   MapValue value:
                -   If parameter's entity type is a
            composite entity:         map from composite
            entity property names to property values     -
            Else: parameter value
        resolved_input (str):
            Final text input which was matched during
            MatchIntent. This value can be different from
            original input sent in request because of
            spelling correction or other processing.
        match_type (google.cloud.dialogflowcx_v3.types.Match.MatchType):
            Type of this [Match][google.cloud.dialogflow.cx.v3.Match].
        confidence (float):
            The confidence of this match. Values range
            from 0.0 (completely uncertain) to 1.0
            (completely certain). This value is for
            informational purpose only and is only used to
            help match the best intent within the
            classification threshold. This value may change
            for the same end-user expression at any time due
            to a model retraining or change in
            implementation.
    """

    class MatchType(proto.Enum):
        r"""Type of a Match."""
        MATCH_TYPE_UNSPECIFIED = 0
        INTENT = 1
        DIRECT_INTENT = 2
        PARAMETER_FILLING = 3
        NO_MATCH = 4
        NO_INPUT = 5
        EVENT = 6

    intent = proto.Field(proto.MESSAGE, number=1, message=gcdc_intent.Intent,)
    event = proto.Field(proto.STRING, number=6,)
    parameters = proto.Field(proto.MESSAGE, number=2, message=struct_pb2.Struct,)
    resolved_input = proto.Field(proto.STRING, number=3,)
    match_type = proto.Field(proto.ENUM, number=4, enum=MatchType,)
    confidence = proto.Field(proto.FLOAT, number=5,)


class MatchIntentRequest(proto.Message):
    r"""Request of [MatchIntent][].
    Attributes:
        session (str):
            Required. The name of the session this query is sent to.
            Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/sessions/<Session ID>``
            or
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/sessions/<Session ID>``.
            If ``Environment ID`` is not specified, we assume default
            'draft' environment. It's up to the API caller to choose an
            appropriate ``Session ID``. It can be a random number or
            some type of session identifiers (preferably hashed). The
            length of the ``Session ID`` must not exceed 36 characters.

            For more information, see the `sessions
            guide <https://cloud.google.com/dialogflow/cx/docs/concept/session>`__.
        query_params (google.cloud.dialogflowcx_v3.types.QueryParameters):
            The parameters of this query.
        query_input (google.cloud.dialogflowcx_v3.types.QueryInput):
            Required. The input specification.
    """

    session = proto.Field(proto.STRING, number=1,)
    query_params = proto.Field(proto.MESSAGE, number=2, message="QueryParameters",)
    query_input = proto.Field(proto.MESSAGE, number=3, message="QueryInput",)


class MatchIntentResponse(proto.Message):
    r"""Response of [MatchIntent][].
    Attributes:
        text (str):
            If [natural language
            text][google.cloud.dialogflow.cx.v3.TextInput] was provided
            as input, this field will contain a copy of the text.
        trigger_intent (str):
            If an [intent][google.cloud.dialogflow.cx.v3.IntentInput]
            was provided as input, this field will contain a copy of the
            intent identifier. Format:
            ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/intents/<Intent ID>``.
        transcript (str):
            If [natural language speech
            audio][google.cloud.dialogflow.cx.v3.AudioInput] was
            provided as input, this field will contain the transcript
            for the audio.
        trigger_event (str):
            If an [event][google.cloud.dialogflow.cx.v3.EventInput] was
            provided as input, this field will contain a copy of the
            event name.
        matches (Sequence[google.cloud.dialogflowcx_v3.types.Match]):
            Match results, if more than one, ordered
            descendingly by the confidence we have that the
            particular intent matches the query.
        current_page (google.cloud.dialogflowcx_v3.types.Page):
            The current [Page][google.cloud.dialogflow.cx.v3.Page].
            Some, not all fields are filled in this message, including
            but not limited to ``name`` and ``display_name``.
    """

    text = proto.Field(proto.STRING, number=1, oneof="query",)
    trigger_intent = proto.Field(proto.STRING, number=2, oneof="query",)
    transcript = proto.Field(proto.STRING, number=3, oneof="query",)
    trigger_event = proto.Field(proto.STRING, number=6, oneof="query",)
    matches = proto.RepeatedField(proto.MESSAGE, number=4, message="Match",)
    current_page = proto.Field(proto.MESSAGE, number=5, message=page.Page,)


class FulfillIntentRequest(proto.Message):
    r"""Request of [FulfillIntent][]
    Attributes:
        match_intent_request (google.cloud.dialogflowcx_v3.types.MatchIntentRequest):
            Must be same as the corresponding MatchIntent
            request, otherwise the behavior is undefined.
        match (google.cloud.dialogflowcx_v3.types.Match):
            The matched intent/event to fulfill.
        output_audio_config (google.cloud.dialogflowcx_v3.types.OutputAudioConfig):
            Instructs the speech synthesizer how to
            generate output audio.
    """

    match_intent_request = proto.Field(
        proto.MESSAGE, number=1, message="MatchIntentRequest",
    )
    match = proto.Field(proto.MESSAGE, number=2, message="Match",)
    output_audio_config = proto.Field(
        proto.MESSAGE, number=3, message=audio_config.OutputAudioConfig,
    )


class FulfillIntentResponse(proto.Message):
    r"""Response of [FulfillIntent][]
    Attributes:
        response_id (str):
            Output only. The unique identifier of the
            response. It can be used to locate a response in
            the training example set or for reporting
            issues.
        query_result (google.cloud.dialogflowcx_v3.types.QueryResult):
            The result of the conversational query.
        output_audio (bytes):
            The audio data bytes encoded as specified in the request.
            Note: The output audio is generated based on the values of
            default platform text responses found in the
            [``query_result.response_messages``][google.cloud.dialogflow.cx.v3.QueryResult.response_messages]
            field. If multiple default text responses exist, they will
            be concatenated when generating audio. If no default
            platform text responses exist, the generated audio content
            will be empty.

            In some scenarios, multiple output audio fields may be
            present in the response structure. In these cases, only the
            top-most-level audio output has content.
        output_audio_config (google.cloud.dialogflowcx_v3.types.OutputAudioConfig):
            The config used by the speech synthesizer to
            generate the output audio.
    """

    response_id = proto.Field(proto.STRING, number=1,)
    query_result = proto.Field(proto.MESSAGE, number=2, message="QueryResult",)
    output_audio = proto.Field(proto.BYTES, number=3,)
    output_audio_config = proto.Field(
        proto.MESSAGE, number=4, message=audio_config.OutputAudioConfig,
    )


class SentimentAnalysisResult(proto.Message):
    r"""The result of sentiment analysis. Sentiment analysis inspects
    user input and identifies the prevailing subjective opinion,
    especially to determine a user's attitude as positive, negative,
    or neutral.

    Attributes:
        score (float):
            Sentiment score between -1.0 (negative
            sentiment) and 1.0 (positive sentiment).
        magnitude (float):
            A non-negative number in the [0, +inf) range, which
            represents the absolute magnitude of sentiment, regardless
            of score (positive or negative).
    """

    score = proto.Field(proto.FLOAT, number=1,)
    magnitude = proto.Field(proto.FLOAT, number=2,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.dialogflow.cx.v3",
    manifest={
        "GetSecuritySettingsRequest",
        "UpdateSecuritySettingsRequest",
        "ListSecuritySettingsRequest",
        "ListSecuritySettingsResponse",
        "CreateSecuritySettingsRequest",
        "DeleteSecuritySettingsRequest",
        "SecuritySettings",
    },
)


class GetSecuritySettingsRequest(proto.Message):
    r"""The request message for
    [SecuritySettingsService.GetSecuritySettings][google.cloud.dialogflow.cx.v3.SecuritySettingsService.GetSecuritySettings].

    Attributes:
        name (str):
            Required. Resource name of the settings. Format:
            ``projects/<Project ID>/locations/<Location ID>/securitySettings/<security settings ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


class UpdateSecuritySettingsRequest(proto.Message):
    r"""The request message for
    [SecuritySettingsService.UpdateSecuritySettings][google.cloud.dialogflow.cx.v3.SecuritySettingsService.UpdateSecuritySettings].

    Attributes:
        security_settings (google.cloud.dialogflowcx_v3.types.SecuritySettings):
            Required. [SecuritySettings] object that contains values for
            each of the fields to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The mask to control which fields
            get updated. If the mask is not present, all
            fields will be updated.
    """

    security_settings = proto.Field(
        proto.MESSAGE, number=1, message="SecuritySettings",
    )
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class ListSecuritySettingsRequest(proto.Message):
    r"""The request message for [SecuritySettings.ListSecuritySettings][].
    Attributes:
        parent (str):
            Required. The location to list all security settings for.
            Format: ``projects/<Project ID>/locations/<Location ID>``.
        page_size (int):
            The maximum number of items to return in a
            single page. By default 20 and at most 100.
        page_token (str):
            The next_page_token value returned from a previous list
            request.
    """

    parent = proto.Field(proto.STRING, number=1,)
    page_size = proto.Field(proto.INT32, number=2,)
    page_token = proto.Field(proto.STRING, number=3,)


class ListSecuritySettingsResponse(proto.Message):
    r"""The response message for [SecuritySettings.ListSecuritySettings][].
    Attributes:
        security_settings (Sequence[google.cloud.dialogflowcx_v3.types.SecuritySettings]):
            The list of security settings.
        next_page_token (str):
            Token to retrieve the next page of results,
            or empty if there are no more results in the
            list.
    """

    @property
    def raw_page(self):
        return self

    security_settings = proto.RepeatedField(
        proto.MESSAGE, number=1, message="SecuritySettings",
    )
    next_page_token = proto.Field(proto.STRING, number=2,)


class CreateSecuritySettingsRequest(proto.Message):
    r"""The request message for [SecuritySettings.CreateSecuritySettings][].
    Attributes:
        parent (str):
            Required. The location to create an
            [SecuritySettings][google.cloud.dialogflow.cx.v3.SecuritySettings]
            for. Format:
            ``projects/<Project ID>/locations/<Location ID>``.
        security_settings (google.cloud.dialogflowcx_v3.types.SecuritySettings):
            Required. The security settings to create.
    """

    parent = proto.Field(proto.STRING, number=1,)
    security_settings = proto.Field(
        proto.MESSAGE, number=2, message="SecuritySettings",
    )


class DeleteSecuritySettingsRequest(proto.Message):
    r"""The request message for [SecuritySettings.DeleteSecuritySettings][].
    Attributes:
        name (str):
            Required. The name of the
            [SecuritySettings][google.cloud.dialogflow.cx.v3.SecuritySettings]
            to delete. Format:
            ``projects/<Project ID>/locations/<Location ID>/securitySettings/<Security Settings ID>``.
    """

    name = proto.Field(proto.STRING, number=1,)


class SecuritySettings(proto.Message):
    r"""Represents the settings related to security issues, such as
    data redaction and data retention. It may take hours for updates
    on the settings to propagate to all the related components and
    take effect.

    Attributes:
        name (str):
            Required. Resource name of the settings. Format:
            ``projects/<Project ID>/locations/<Location ID>/securitySettings/<Security Settings ID>``.
        display_name (str):
            Required. The human-readable name of the
            security settings, unique within the location.
        redaction_strategy (google.cloud.dialogflowcx_v3.types.SecuritySettings.RedactionStrategy):
            Strategy that defines how we do redaction.
        redaction_scope (google.cloud.dialogflowcx_v3.types.SecuritySettings.RedactionScope):
            Defines the data for which Dialogflow applies
            redaction. Dialogflow does not redact data that
            it does not have access to – for example, Cloud
            logging.
        inspect_template (str):
            DLP inspect template name. Use this template to define
            inspect base settings.

            If empty, we use the default DLP inspect config.

            The template name will have one of the following formats:
            ``projects/<Project ID>/inspectTemplates/<Template ID>`` OR
            ``projects/<Project ID>/locations/<Location ID>/inspectTemplates/<Template ID>``
            OR
            ``organizations/<Organization ID>/inspectTemplates/<Template ID>``
        retention_window_days (int):
            Retains data in interaction logging for the
            specified number of days. This does not apply to
            Cloud logging, which is owned by the user - not
            Dialogflow.
            User must Set a value lower than Dialogflow's
            default 30d TTL. Setting a value higher than
            that has no effect.
            A missing value or setting to 0 also means we
            use Dialogflow's default TTL.
            Note: Interaction logging is a limited access
            feature. Talk to your Google representative to
            check availability for you.
        purge_data_types (Sequence[google.cloud.dialogflowcx_v3.types.SecuritySettings.PurgeDataType]):
            List of types of data to remove when
            retention settings triggers purge.
    """

    class RedactionStrategy(proto.Enum):
        r"""Defines how we redact data."""
        REDACTION_STRATEGY_UNSPECIFIED = 0
        REDACT_WITH_SERVICE = 1

    class RedactionScope(proto.Enum):
        r"""Defines what types of data to redact."""
        REDACTION_SCOPE_UNSPECIFIED = 0
        REDACT_DISK_STORAGE = 2

    class PurgeDataType(proto.Enum):
        r"""Type of data we purge after retention settings triggers
        purge.
        """
        PURGE_DATA_TYPE_UNSPECIFIED = 0
        DIALOGFLOW_HISTORY = 1

    name = proto.Field(proto.STRING, number=1,)
    display_name = proto.Field(proto.STRING, number=2,)
    redaction_strategy = proto.Field(proto.ENUM, number=3, enum=RedactionStrategy,)
    redaction_scope = proto.Field(proto.ENUM, number=4, enum=RedactionScope,)
    inspect_template = proto.Field(proto.STRING, number=9,)
    retention_window_days = proto.Field(proto.INT32, number=6, oneof="data_retention",)
    purge_data_types = proto.RepeatedField(proto.ENUM, number=8, enum=PurgeDataType,)


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Sequence, Tuple, Type, Union
import pkg_resources

import google.api_core.client_options as ClientOptions  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.dialogflowcx_v3beta1.services.experiments import pagers
from google.cloud.dialogflowcx_v3beta1.types import experiment
from google.cloud.dialogflowcx_v3beta1.types import experiment as gcdc_experiment
from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from .transports.base import ExperimentsTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import ExperimentsGrpcAsyncIOTransport
from .client import ExperimentsClient


class ExperimentsAsyncClient:
    """Service for managing
    [Experiments][google.cloud.dialogflow.cx.v3beta1.Experiment].
    """

    _client: ExperimentsClient

    DEFAULT_ENDPOINT = ExperimentsClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = ExperimentsClient.DEFAULT_MTLS_ENDPOINT

    experiment_path = staticmethod(ExperimentsClient.experiment_path)
    parse_experiment_path = staticmethod(ExperimentsClient.parse_experiment_path)
    version_path = staticmethod(ExperimentsClient.version_path)
    parse_version_path = staticmethod(ExperimentsClient.parse_version_path)
    common_billing_account_path = staticmethod(
        ExperimentsClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        ExperimentsClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(ExperimentsClient.common_folder_path)
    parse_common_folder_path = staticmethod(ExperimentsClient.parse_common_folder_path)
    common_organization_path = staticmethod(ExperimentsClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        ExperimentsClient.parse_common_organization_path
    )
    common_project_path = staticmethod(ExperimentsClient.common_project_path)
    parse_common_project_path = staticmethod(
        ExperimentsClient.parse_common_project_path
    )
    common_location_path = staticmethod(ExperimentsClient.common_location_path)
    parse_common_location_path = staticmethod(
        ExperimentsClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ExperimentsAsyncClient: The constructed client.
        """
        return ExperimentsClient.from_service_account_info.__func__(ExperimentsAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            ExperimentsAsyncClient: The constructed client.
        """
        return ExperimentsClient.from_service_account_file.__func__(ExperimentsAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> ExperimentsTransport:
        """Returns the transport used by the client instance.

        Returns:
            ExperimentsTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(ExperimentsClient).get_transport_class, type(ExperimentsClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, ExperimentsTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the experiments client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.ExperimentsTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = ExperimentsClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_experiments(
        self,
        request: experiment.ListExperimentsRequest = None,
        *,
        parent: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListExperimentsAsyncPager:
        r"""Returns the list of all experiments in the specified
        [Environment][google.cloud.dialogflow.cx.v3beta1.Environment].

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.ListExperimentsRequest`):
                The request object. The request message for
                [Experiments.ListExperiments][google.cloud.dialogflow.cx.v3beta1.Experiments.ListExperiments].
            parent (:class:`str`):
                Required. The
                [Environment][google.cloud.dialogflow.cx.v3beta1.Environment]
                to list all environments for. Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.services.experiments.pagers.ListExperimentsAsyncPager:
                The response message for
                [Experiments.ListExperiments][google.cloud.dialogflow.cx.v3beta1.Experiments.ListExperiments].

                Iterating over this object will yield results and
                resolve additional pages automatically.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = experiment.ListExperimentsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_experiments,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListExperimentsAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_experiment(
        self,
        request: experiment.GetExperimentRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> experiment.Experiment:
        r"""Retrieves the specified
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment].

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.GetExperimentRequest`):
                The request object. The request message for
                [Experiments.GetExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.GetExperiment].
            name (:class:`str`):
                Required. The name of the
                [Environment][google.cloud.dialogflow.cx.v3beta1.Environment].
                Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/experiments/<Experiment ID>``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.types.Experiment:
                Represents an experiment in an
                environment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = experiment.GetExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def create_experiment(
        self,
        request: gcdc_experiment.CreateExperimentRequest = None,
        *,
        parent: str = None,
        experiment: gcdc_experiment.Experiment = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> gcdc_experiment.Experiment:
        r"""Creates an
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment] in
        the specified
        [Environment][google.cloud.dialogflow.cx.v3beta1.Environment].

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.CreateExperimentRequest`):
                The request object. The request message for
                [Experiments.CreateExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.CreateExperiment].
            parent (:class:`str`):
                Required. The
                [Agent][google.cloud.dialogflow.cx.v3beta1.Agent] to
                create an
                [Environment][google.cloud.dialogflow.cx.v3beta1.Environment]
                for. Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>``.

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            experiment (:class:`google.cloud.dialogflowcx_v3beta1.types.Experiment`):
                Required. The experiment to create.
                This corresponds to the ``experiment`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.types.Experiment:
                Represents an experiment in an
                environment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent, experiment])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = gcdc_experiment.CreateExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent
        if experiment is not None:
            request.experiment = experiment

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.create_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_experiment(
        self,
        request: gcdc_experiment.UpdateExperimentRequest = None,
        *,
        experiment: gcdc_experiment.Experiment = None,
        update_mask: field_mask_pb2.FieldMask = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> gcdc_experiment.Experiment:
        r"""Updates the specified
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment].

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.UpdateExperimentRequest`):
                The request object. The request message for
                [Experiments.UpdateExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.UpdateExperiment].
            experiment (:class:`google.cloud.dialogflowcx_v3beta1.types.Experiment`):
                Required. The experiment to update.
                This corresponds to the ``experiment`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                Required. The mask to control which
                fields get updated.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.types.Experiment:
                Represents an experiment in an
                environment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([experiment, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = gcdc_experiment.UpdateExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if experiment is not None:
            request.experiment = experiment
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("experiment.name", request.experiment.name),)
            ),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def delete_experiment(
        self,
        request: experiment.DeleteExperimentRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the specified
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment].

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.DeleteExperimentRequest`):
                The request object. The request message for
                [Experiments.DeleteExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.DeleteExperiment].
            name (:class:`str`):
                Required. The name of the
                [Environment][google.cloud.dialogflow.cx.v3beta1.Environment]
                to delete. Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/experiments/<Experiment ID>``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = experiment.DeleteExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

    async def start_experiment(
        self,
        request: experiment.StartExperimentRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> experiment.Experiment:
        r"""Starts the specified
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment].
        This rpc only changes the state of experiment from PENDING to
        RUNNING.

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.StartExperimentRequest`):
                The request object. The request message for
                [Experiments.StartExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.StartExperiment].
            name (:class:`str`):
                Required. Resource name of the experiment to start.
                Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/experiments/<Experiment ID>``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.types.Experiment:
                Represents an experiment in an
                environment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = experiment.StartExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.start_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def stop_experiment(
        self,
        request: experiment.StopExperimentRequest = None,
        *,
        name: str = None,
        retry: retries.Retry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> experiment.Experiment:
        r"""Stops the specified
        [Experiment][google.cloud.dialogflow.cx.v3beta1.Experiment].
        This rpc only changes the state of experiment from RUNNING to
        DONE.

        Args:
            request (:class:`google.cloud.dialogflowcx_v3beta1.types.StopExperimentRequest`):
                The request object. The request message for
                [Experiments.StopExperiment][google.cloud.dialogflow.cx.v3beta1.Experiments.StopExperiment].
            name (:class:`str`):
                Required. Resource name of the experiment to stop.
                Format:
                ``projects/<Project ID>/locations/<Location ID>/agents/<Agent ID>/environments/<Environment ID>/experiments/<Experiment ID>``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.dialogflowcx_v3beta1.types.Experiment:
                Represents an experiment in an
                environment.

        """
        # Create or coerce a protobuf request object.
        # Sanity check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = experiment.StopExperimentRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.stop_experiment,
            default_timeout=None,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-dialogflowcx",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("ExperimentsAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import abc
from typing import Awaitable, Callable, Dict, Optional, Sequence, Union
import packaging.version
import pkg_resources

import google.auth  # type: ignore
import google.api_core  # type: ignore
from google.api_core import exceptions as core_exceptions  # type: ignore
from google.api_core import gapic_v1  # type: ignore
from google.api_core import retry as retries  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.cloud.dialogflowcx_v3.types import intent
from google.cloud.dialogflowcx_v3.types import intent as gcdc_intent
from google.protobuf import empty_pb2  # type: ignore

try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-dialogflowcx",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()

try:
    # google.auth.__version__ was added in 1.26.0
    _GOOGLE_AUTH_VERSION = google.auth.__version__
except AttributeError:
    try:  # try pkg_resources if it is available
        _GOOGLE_AUTH_VERSION = pkg_resources.get_distribution("google-auth").version
    except pkg_resources.DistributionNotFound:  # pragma: NO COVER
        _GOOGLE_AUTH_VERSION = None


class IntentsTransport(abc.ABC):
    """Abstract transport class for Intents."""

    AUTH_SCOPES = (
        "https://www.googleapis.com/auth/cloud-platform",
        "https://www.googleapis.com/auth/dialogflow",
    )

    DEFAULT_HOST: str = "dialogflow.googleapis.com"

    def __init__(
        self,
        *,
        host: str = DEFAULT_HOST,
        credentials: ga_credentials.Credentials = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        **kwargs,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A list of scopes.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
        """
        # Save the hostname. Default to port 443 (HTTPS) if none is specified.
        if ":" not in host:
            host += ":443"
        self._host = host

        scopes_kwargs = self._get_scopes_kwargs(self._host, scopes)

        # Save the scopes.
        self._scopes = scopes

        # If no credentials are provided, then determine the appropriate
        # defaults.
        if credentials and credentials_file:
            raise core_exceptions.DuplicateCredentialArgs(
                "'credentials_file' and 'credentials' are mutually exclusive"
            )

        if credentials_file is not None:
            credentials, _ = google.auth.load_credentials_from_file(
                credentials_file, **scopes_kwargs, quota_project_id=quota_project_id
            )

        elif credentials is None:
            credentials, _ = google.auth.default(
                **scopes_kwargs, quota_project_id=quota_project_id
            )

        # If the credentials is service account credentials, then always try to use self signed JWT.
        if (
            always_use_jwt_access
            and isinstance(credentials, service_account.Credentials)
            and hasattr(service_account.Credentials, "with_always_use_jwt_access")
        ):
            credentials = credentials.with_always_use_jwt_access(True)

        # Save the credentials.
        self._credentials = credentials

    # TODO(busunkim): This method is in the base transport
    # to avoid duplicating code across the transport classes. These functions
    # should be deleted once the minimum required versions of google-auth is increased.

    # TODO: Remove this function once google-auth >= 1.25.0 is required
    @classmethod
    def _get_scopes_kwargs(
        cls, host: str, scopes: Optional[Sequence[str]]
    ) -> Dict[str, Optional[Sequence[str]]]:
        """Returns scopes kwargs to pass to google-auth methods depending on the google-auth version"""

        scopes_kwargs = {}

        if _GOOGLE_AUTH_VERSION and (
            packaging.version.parse(_GOOGLE_AUTH_VERSION)
            >= packaging.version.parse("1.25.0")
        ):
            scopes_kwargs = {"scopes": scopes, "default_scopes": cls.AUTH_SCOPES}
        else:
            scopes_kwargs = {"scopes": scopes or cls.AUTH_SCOPES}

        return scopes_kwargs

    def _prep_wrapped_messages(self, client_info):
        # Precompute the wrapped methods.
        self._wrapped_methods = {
            self.list_intents: gapic_v1.method.wrap_method(
                self.list_intents, default_timeout=None, client_info=client_info,
            ),
            self.get_intent: gapic_v1.method.wrap_method(
                self.get_intent, default_timeout=None, client_info=client_info,
            ),
            self.create_intent: gapic_v1.method.wrap_method(
                self.create_intent, default_timeout=None, client_info=client_info,
            ),
            self.update_intent: gapic_v1.method.wrap_method(
                self.update_intent, default_timeout=None, client_info=client_info,
            ),
            self.delete_intent: gapic_v1.method.wrap_method(
                self.delete_intent, default_timeout=None, client_info=client_info,
            ),
        }

    @property
    def list_intents(
        self,
    ) -> Callable[
        [intent.ListIntentsRequest],
        Union[intent.ListIntentsResponse, Awaitable[intent.ListIntentsResponse]],
    ]:
        raise NotImplementedError()

    @property
    def get_intent(
        self,
    ) -> Callable[
        [intent.GetIntentRequest], Union[intent.Intent, Awaitable[intent.Intent]]
    ]:
        raise NotImplementedError()

    @property
    def create_intent(
        self,
    ) -> Callable[
        [gcdc_intent.CreateIntentRequest],
        Union[gcdc_intent.Intent, Awaitable[gcdc_intent.Intent]],
    ]:
        raise NotImplementedError()

    @property
    def update_intent(
        self,
    ) -> Callable[
        [gcdc_intent.UpdateIntentRequest],
        Union[gcdc_intent.Intent, Awaitable[gcdc_intent.Intent]],
    ]:
        raise NotImplementedError()

    @property
    def delete_intent(
        self,
    ) -> Callable[
        [intent.DeleteIntentRequest], Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]]
    ]:
        raise NotImplementedError()


__all__ = ("IntentsTransport",)
